#  Copyright 2021-2022 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from typing import List, Sequence, Optional

from PIL import Image
from range_typed_integers import *


class BpaFrameInfo:
    duration_per_frame: u16
    unk2: u16
    def __init__(self, duration_per_frame: int, unk2: int): ...


class Bpa:
    number_of_tiles: u16
    number_of_frames: u16
    tiles: Sequence[bytes]
    frame_info: Sequence[BpaFrameInfo]

    def __init__(self, data: bytes): ...
    @classmethod
    def new_empty(cls) -> 'Bpa': ...
    def get_tile(self, tile_idx: int, frame_idx: int) -> bytes: ...
    def tiles_to_pil_separate(self, palette: Sequence[int], width_in_tiles: int = 20) -> List[Image.Image]: ...
    def tiles_to_pil(self, palette: Sequence[int]) -> Optional[Image.Image]: ...
    def pil_to_tiles(self, image: Image.Image) -> None: ...
    def pil_to_tiles_separate(self, images: List[Image.Image]) -> None: ...
    def tiles_for_frame(self, frame: int) -> Sequence[bytes]: ...


class BpaWriter:
    def __new__(cls): ...
    def write(self, model: Bpa) -> bytes: ...



#  Copyright 2020-2023 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from typing import Sequence

from range_typed_integers import *

COUNT_GLOBAL_VARS: u32
COUNT_LOCAL_VARS: u32
DEFINITION_STRUCT_SIZE: int


class ScriptVariableDefinition:
    id: int
    type: u16
    unk1: u16
    memoffset: u16
    bitshift: u16
    nbvalues: u16
    default: i16
    name: str


class ScriptVariableTables:
    globals: Sequence[ScriptVariableDefinition]
    locals: Sequence[ScriptVariableDefinition]

    def __init__(self, arm9: bytes, global_start: u32, local_start: u32, subtract_from_name_addrs: u32):
        ...



#  Copyright 2020-2022 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from typing import Sequence

from PIL import Image


class Dpci:
    tiles: Sequence[bytes]

    def __init__(self, data: bytes): ...

    def tiles_to_pil(self, palettes: Sequence[Sequence[int]], width_in_tiles=20, palette_index=0) -> Image.Image: ...

    def pil_to_tiles(self, image: Image.Image): ...

    def import_tiles(self, tiles: Sequence[bytearray], contains_null_tile=False): ...


class DpciWriter:
    def __new__(cls): ...
    def write(self, model: Dpci) -> bytes: ...



#  Copyright 2020-2022 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from typing import Sequence, List

from PIL import Image

from skytemple_rust import TilemapEntry


class Bgp:
    palettes: Sequence[Sequence[int]] = []
    tiles: Sequence[bytearray] = []
    tilemap: Sequence[TilemapEntry] = []

    def __init__(self, data: bytes): ...

    def to_pil(self, ignore_flip_bits=False) -> Image.Image: ...

    def from_pil(self, pil: Image.Image, force_import=False) -> None: ...


class BgpWriter:
    def __new__(cls): ...
    def write(self, model: Bgp) -> bytes: ...



#  Copyright 2021-2022 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from typing import List, Optional, Protocol, Union, Sequence

from skytemple_rust.st_bma import Bma
from skytemple_rust.st_bpa import Bpa
from skytemple_rust.st_bpc import Bpc
from skytemple_rust.st_bpl import Bpl


class RomFileProviderProtocol(Protocol):
    def getFileByName(self, filename: str) -> bytes: ...


class BgListEntry:
    bpl_name: str
    bpc_name: str
    bma_name: str
    bpa_names: Sequence[Optional[str]]
    def __init__(self, bpl_name: str, bpc_name: str, bma_name: str, bpa_names: List[Optional[str]]): ...
    def get_bpl(self, rom_or_directory_root: Union[str, RomFileProviderProtocol]) -> Bpl: ...
    def get_bpc(self, rom_or_directory_root: Union[str, RomFileProviderProtocol], bpc_tiling_width: int = 3, bpc_tiling_height: int = 3) -> Bpc: ...
    def get_bma(self, rom_or_directory_root: Union[str, RomFileProviderProtocol]) -> Bma: ...
    def get_bpas(self, rom_or_directory_root: Union[str, RomFileProviderProtocol]) -> List[Optional[Bpa]]: ...


class BgList(BgListEntry):
    level: Sequence[BgListEntry]

    def __init__(self, data: bytes): ...
    def find_bma(self, name: str) -> int: ...
    def find_bpl(self, name: str) -> int: ...
    def find_bpc(self, name: str) -> int: ...
    def find_bpa(self, name: str) -> int: ...
    def add_level(self, level: BgListEntry): ...
    def set_level(self, level_id: int, level: BgListEntry): ...
    def set_level_bpa(self, level_id: int, bpa_id: int, bpa_name: Optional[str]): ...

class BgListWriter:
    def __new__(cls): ...
    def write(self, model: BgList) -> bytes: ...



#  Copyright 2020-2022 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from typing import Sequence
from PIL import Image

from skytemple_rust.st_dpc import Dpc
from skytemple_rust.st_dpci import Dpci

_DmaType = int
    # WALL = 0
    # WATER = 1
    # FLOOR = 2


_DmaExtraType = int
    # FLOOR1 = 0
    # WALL_OR_VOID = 1
    # FLOOR2 = 2


class Dma:
    chunk_mappings: Sequence[int]

    def __init__(self, data: bytes): ...

    def get(self, get_type: _DmaType, neighbors_same: int) -> Sequence[int]: ...

    def get_extra(self, extra_type: _DmaExtraType) -> Sequence[int]: ...

    def set(self, get_type: _DmaType, neighbors_same: int, variation_index: int, value: int): ...

    def set_extra(self, extra_type: _DmaExtraType, index: int, value: int): ...

    def to_pil(
            self, dpc: Dpc, dpci: Dpci, palettes: Sequence[Sequence[int]]
    ) -> Image.Image: ...


class DmaWriter:
    def __new__(cls): ...
    def write(self, model: Dma) -> bytes: ...



from range_typed_integers import u16


class TilemapEntry:
    idx: int
    flip_x: bool
    flip_y: bool
    pal_idx: int

    def __init__(self, idx: int, flip_x: bool, flip_y: bool, pal_idx: int, ignore_too_large: bool = False): ...
    def to_int(self) -> u16: ...
    @classmethod
    def from_int(cls, entry: u16) -> TilemapEntry: ...



#  Copyright 2020-2022 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from typing import Sequence, List

from skytemple_rust.st_sir0 import Sir0Serializable


class Dpla(Sir0Serializable):
    colors: Sequence[Sequence[int]]
    durations_per_frame_for_colors: Sequence[int]

    def __init__(self, data: bytes, pointer_to_pointers: int): ...

    def get_palette_for_frame(self, pal_idx: int, frame_id: int) -> List[int]: ...

    def has_for_palette(self, palette_idx: int) -> bool: ...

    def get_frame_count_for_palette(self, palette_idx: int) -> int: ...

    def enable_for_palette(self, palid: int) -> None: ...

    def disable_for_palette(self, palid: int) -> None: ...

    def get_duration_for_palette(self, palette_idx: int) -> int: ...

    def set_duration_for_palette(self, palid: int, duration: int) -> None: ...

    def apply_palette_animations(self, palettes: Sequence[Sequence[int]], frame_idx: int) -> List[List[int]]: ...
